@@ -26,11 +26,13 @@ class Device(models.Model):
     OFFLINE = 0
     IDLE = 1
     RUNNING = 2
+    OFFLINING = 3
 
     STATUS_CHOICES = (
         (OFFLINE, 'Offline'),
         (IDLE, 'Idle'),
         (RUNNING, 'Running'),
+        (OFFLINING, 'Going offline'),
     )
 
     hostname = models.CharField(
@@ -51,6 +53,20 @@ class Device(models.Model):
         verbose_name = _(u"Device status"),
     )
 
+    def can_admin(self, user):
+        return user.has_perm('lava_scheduler_app.change_device')
+
+    def put_into_maintenance_mode(self):
+        if self.status == self.RUNNING:
+            self.status = self.OFFLINING
+        else:
+            self.status = self.OFFLINE
+        self.save()
+
+    def put_into_online_mode(self):
+        self.status = self.IDLE
+        self.save()
+
     def __unicode__(self):
         return self.hostname
 
@@ -69,6 +85,7 @@ class TestJob(models.Model):
     COMPLETE = 2
     INCOMPLETE = 3
     CANCELED = 4
+    CANCELING = 5
 
     STATUS_CHOICES = (
         (SUBMITTED, 'Submitted'),
@@ -76,6 +93,7 @@ class TestJob(models.Model):
         (COMPLETE, 'Complete'),
         (INCOMPLETE, 'Incomplete'),
         (CANCELED, 'Canceled'),
+        (CANCELING, 'Canceling'),
     )
 
     id = models.AutoField(primary_key=True)
@@ -92,13 +110,13 @@ class TestJob(models.Model):
 
     # Only one of these two should be non-null.
     requested_device = models.ForeignKey(
-        Device, null=True, default=None, related_name='+')
+        Device, null=True, default=None, related_name='+', blank=True)
     requested_device_type = models.ForeignKey(
-        DeviceType, null=True, default=None, related_name='+')
+        DeviceType, null=True, default=None, related_name='+', blank=True)
 
     # This is set once the job starts.
     actual_device = models.ForeignKey(
-        Device, null=True, default=None, related_name='+')
+        Device, null=True, default=None, related_name='+', blank=True)
 
     #priority = models.IntegerField(
     #    verbose_name = _(u"Priority"),
@@ -133,7 +151,7 @@ class TestJob(models.Model):
         editable = False,
     )
     log_file = models.FileField(
-        upload_to='lava-logs', default=None, null=True)
+        upload_to='lava-logs', default=None, null=True, blank=True)
 
     results_link = models.CharField(
         max_length=400, default=None, null=True, blank=True)
@@ -158,3 +176,13 @@ class TestJob(models.Model):
             requested_device_type=device_type)
         job.save()
         return job
+
+    def can_cancel(self, user):
+        return user.is_superuser or user == self.submitter
+
+    def cancel(self):
+        if self.status == TestJob.RUNNING:
+            self.status = TestJob.CANCELING
+        else:
+            self.status = TestJob.CANCELED
+        self.save()
